#include "StdAfx.h"
#include "Mpeg2SequenceFilter.h"

CMpeg2SequenceFilter::CMpeg2SequenceFilter(LPUNKNOWN pUnk, HRESULT *phr)
	: CTransformFilter(MPEG2SEQUENCEFILTER_NAME, pUnk, CLSID_MPEG2SEQFILTER)
	,m_Mpeg2Parser(&m_Mpeg2Sequence)
	,m_Mpeg2Sequence(this)	
	,m_VideoInfo()
{
	m_pfnVideoInfoRecvFunc=NULL;
	m_pCallbackParam=NULL;
}

CMpeg2SequenceFilter::~CMpeg2SequenceFilter(void)
{

}

CUnknown * WINAPI CMpeg2SequenceFilter::CreateInstance(LPUNKNOWN pUnk, HRESULT *phr)
{
	// CX^X쐬
	CMpeg2SequenceFilter *pNewFilter = new CMpeg2SequenceFilter(pUnk, phr);
	if(!pNewFilter)*phr = E_OUTOFMEMORY;
	
	return dynamic_cast<CUnknown *>(pNewFilter);
}


HRESULT CMpeg2SequenceFilter::CheckInputType(const CMediaType* mtIn)
{
    CheckPointer(mtIn, E_POINTER);

	if(*mtIn->Type() == MEDIATYPE_Video){
		if(*mtIn->Subtype() == MEDIASUBTYPE_MPEG2_VIDEO){

				return S_OK;
			}
		}
	return VFW_E_TYPE_NOT_ACCEPTED;
}

HRESULT CMpeg2SequenceFilter::CheckTransform(const CMediaType* mtIn, const CMediaType* mtOut)
{
    CheckPointer(mtIn, E_POINTER);
    CheckPointer(mtOut, E_POINTER);
    
	if(*mtOut->Type() == MEDIATYPE_Video){
		if(*mtOut->Subtype() == MEDIASUBTYPE_MPEG2_VIDEO){			
			return S_OK;
			}
		}
    
	return VFW_E_TYPE_NOT_ACCEPTED;
}

HRESULT CMpeg2SequenceFilter::CompleteConnect(PIN_DIRECTION direction,IPin *pReceivePin)
{
	return S_OK;
}

HRESULT CMpeg2SequenceFilter::DecideBufferSize(IMemAllocator * pAllocator, ALLOCATOR_PROPERTIES *pprop)
{
	CAutoLock AutoLock(m_pLock);
	CheckPointer(pAllocator, E_POINTER);
	CheckPointer(pprop, E_POINTER);

	// obt@1΂悢
	if(!pprop->cBuffers)pprop->cBuffers = 1L;

	// Ƃ肠1MBm
	if(pprop->cbBuffer < 0x100000L)pprop->cbBuffer = 0x100000L;

	// AP[^vpeBݒ肵Ȃ
	ALLOCATOR_PROPERTIES Actual;
	HRESULT hr = pAllocator->SetProperties(pprop, &Actual);
	if(FAILED(hr))return hr;

	// v󂯓ꂽ
	if(Actual.cbBuffer < pprop->cbBuffer)return E_FAIL;

	return S_OK;
}


HRESULT CMpeg2SequenceFilter::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	CAutoLock AutoLock(m_pLock);
	CheckPointer(pMediaType, E_POINTER);

	if(iPosition < 0)return E_INVALIDARG;
	if(iPosition > 0)return VFW_S_NO_MORE_ITEMS;
	*pMediaType=this->m_pInput->CurrentMediaType();
	return S_OK;
}

HRESULT CMpeg2SequenceFilter::StartStreaming(void)
{
	return S_OK;
}

HRESULT CMpeg2SequenceFilter::StopStreaming(void)
{
	return S_OK;
}


HRESULT CMpeg2SequenceFilter::Transform(IMediaSample *pIn, IMediaSample *pOut)
{
	// ̓f[^|C^擾
	BYTE *pInData = NULL;
	pIn->GetPointer(&pInData);
	long lDataSize = pIn->GetActualDataLength();

	// o̓|C^擾
	pOut->SetActualDataLength(0);
	const DWORD dwOffset = pOut->GetActualDataLength();
	BYTE *pOutBuff = NULL;
	pOut->GetPointer(&pOutBuff);
	pOutBuff = &pOutBuff[dwOffset];

	// V[PX擾
	m_Mpeg2Parser.StoreEs(pInData,lDataSize);

	// ̂܂܏o͂ɃRs[
	::CopyMemory(pOutBuff,pInData,lDataSize);
	pOut->SetActualDataLength(dwOffset + lDataSize);
	return S_OK;
}

void CMpeg2SequenceFilter::SetRecvCallback(MPEG2SEQUENCE_VIDEOINFO_FUNC pCallback, const PVOID pParam)
{
	m_pfnVideoInfoRecvFunc = pCallback;
	m_pCallbackParam = pParam;
}

void CMpeg2SequenceFilter::OnSequence(const CMpeg2Sequence *pSequence)
{
	BYTE byX=0,byY=0;
	WORD wX,wY;
	switch(pSequence->GetAspectRatioInfo()){
	case 2:
		byX = 4;
		byY = 3;
		break;
	case 3:
		byX = 16;
		byY = 9;
		break;
	case 4:
		byX = 221;
		byY = 100;
		break;
		}
	wX = pSequence->GetHorizontalSize();
	wY = pSequence->GetVerticalSize();

	if(wX!=m_VideoInfo.wWidth || wY!=m_VideoInfo.wHeight || byX!=m_VideoInfo.byAspectRatioX || byY!=m_VideoInfo.byAspectRatioY){
		// fTCYς
		m_VideoInfo.wWidth = wX;
		m_VideoInfo.wHeight= wY;
		m_VideoInfo.byAspectRatioX = byX;
		m_VideoInfo.byAspectRatioY = byY;
		// ʒm
		if(m_pfnVideoInfoRecvFunc) m_pfnVideoInfoRecvFunc(&m_VideoInfo,m_pCallbackParam);
		}
}

const bool CMpeg2SequenceFilter::GetVideoSize(WORD *pX, WORD *pY)
{
	if(m_VideoInfo.wWidth==0||m_VideoInfo.wHeight==0) return false;
	if(pX) *pX = m_VideoInfo.wWidth;
	if(pY) *pY = m_VideoInfo.wHeight;
	return true;
}

const bool CMpeg2SequenceFilter::GetAspectRatio(BYTE *pX,BYTE *pY)
{
	if(m_VideoInfo.byAspectRatioX==0||m_VideoInfo.byAspectRatioY==0) return false;
	if(pX) *pX = m_VideoInfo.byAspectRatioX;
	if(pY) *pY = m_VideoInfo.byAspectRatioY;
	return true;
}